/* <a href="http://www.ooRexx.org>Object Rexx</a> convenience class for making IBM's and
   Apache's Bean Scripting Framework (BSF) available in an OO-style as well: Java classes and
   Java objects appear as Object Rexx classes and objects and one can send Object Rexx
   messages to them.

   Object Rexx programmers should appreciate this little class ...
   ;)

   date:
          2006-01-06, rgf: - now fixed bsf.import to use the routine bsf.loadClass(), changed the
                             code to use the new class method bsf.setBSF4RexxInfos to store the
                             Java class object proxy and save the fully qualified Java class name;
                             added public routines pp() and iif()

          2006-01-05, rgf: - now using the much better BSF-subfunction "loadClass" instead of the
                             bad variant "Class~forName()", added class method "bsf.loadClass" to "BSF",
                             and routine "bsf.loadClass()"

          2005-08-30, rgf: - now also assigning the beanName (index into BSF registry) as objectName
                             for Java class objects; this way no special consideration has to be
                             applied if JCOs are used as arguments; the ooRexx classes reveal the
                             fully qualified Java class name either via the defaultName-method, via
                             the class attribute "bsf.FullJavaClassName" or by sending them the
                             "toString()" message;
                             license change to Apache v2.0 and CPLv1.0

          2005-07-30, rgf: - fixed a bug in creation of array: now pre-registered names (entries in .bsf4rexx)
                             and any other Java class object proxy can be used to indicate component type for the
                             Java array

          2005-07-10, rgf: - added public routines bsf.lookupBean(), bsf.unregisterBean(),
                             bsf.pollEventText(), bsf.postEventText(), bsf.getStaticValue(),
                             bsf.getStaticValueStrict() and synonym bsf.getConstant(),
                             bsf.createArray(): makes it easier to code than the
                             versions ".bsf~bsf.xyz()"

          2005-07-06, rgf: - renamed "bsf.checkResult" into "bsf.wrap"

          2005-07-04, rgf: - corrected bug in MAKEARRAY: now the individual elements are wrapped
                             in BSF_REFERENCE, if they represent Java objects

          2005-06-07, rgf: - renamed "bsf.box" to "box" and "bsf.unbox" to "unbox"

          2005-06-06, rgf: - bsf.import changed: if second argument is .nil or the empty string,
                             then - and only then - there will be no entry in .local; if 2nd
                             argument is omitted, then the Java class name is used as the name
                             on ooRexx

          2005-06-05, rgf: - renamed BSF-pass through class methods to begin with "BSF.",
                           - using pre-registered Java-class proxies for box() and unbox(),
                           changed bsf.import logic to allow for dynamic allocating (if a
                           Java object proxy needs to access its Java class object):
                           - added instance method "bsf.class" to return the Java class object
                           proxy, which allows accessing the static methods and fields as well;


          2005-05-07, rgf: added bsf.getFieldValueStrict (strict case!),
                      methods having a trailing "=" are interpreted as setter methods, causing
                      the "setFieldValue" subfunction to be invoked using the method name without
                      the trailing equal sign as the name of the field

          2005-05-02, rgf, imported the primitive type wrapper classes into ooRexx
                           added box() and unbox(),
                           removed .box.true and .box.false (boxed as Booleans), -- 2005-05-03, rgf

          2004-03-04, rgf, test if BSFLoadJava() is registered before invoking it
          2004-02-22, rgf, trap exception on registering external Rexx-functions; needed
                      for Tobias Specht for debugging BWS on Konqueror;
                      partially removing a reported bug by TS (ORX not able handling literals
                      larger than 250 chars in INTERPRET)

          2003-11-19, rgf, added "RAISE PROPAGATE" statements in order for Rexx programs
                      being able to intercept exceptions which may occur in programs using
                      the proxy classes of this file

          2003-09-05, rgf, added "newStrict" floating constructor method for imported
                      Java classes and "addNewStrict" class method to attach it to
                      Object Rexx proxy classes by the "import" class method (i.e. setting
                      up an Object Rexx class as a proxy), to allow using strictly typed
                      arguments at instantiation time

          2003-06-01, rgf, added Apache license

          2003-05-10, rgf, changed "_new" to "Strict", changed arrays to never
                      use strict typing (not necessary as type can be inferred
                      from array itself)

          2003-05-07, rgf, using the new "invoke_new", which allows for not
                      using type information with the arguments

          2003-03-10, rgf, changed ".bsf.cls" to ".bsf4rexx" and ".JC" to ".class"
          2003-02-09, rgf, no dimensional limits on arrays anymore,
                           makearray(), supplier() now created at the Java side, merely re-using

          2003-01-29, Augsburg-->Vienna (train), adding direct JVM-loading support,
                      BSF_PROXY, .BSF.cls (for accessing pre-registered Java class objects)

          2003-01-28, Augsburg, renamed "BSF_ARRAY" to "BSF_ARRAY_PROXY" to denote its purpose
          2001-04-22, 2001-05-02 (on the airplane, returning home
                                  from the 12th International Rexx Symposium),
          2001-05-07 (fixed bug in bsf:init, if directly used to instantiate data)
          2001-05-26 (made method "bsf.exit" available at the instance level)
          2001-06-02 (BSF: made method "bsf.sleep" available at the instance level and
                      as "sleep" at the class level;
                      BSF_ARRAY_PROXY: MAKEARRAY and SUPPLIER now only return entries with
                                 values as does Object Rexx with native arrays)

   version:     1.4.1

   authors:     (c) 2001-2006 Rony G. Flatscher, University of Augsburg, University of Essen,
                    WU (pronounced: "vey-uh") University Vienna
                         Peter Kalender (October 2000 - February 2001), University of Essen, feasibility study

  If this program is part of a distribution of an Apache foundation project (e.g. BSF,
  the Bean Scripting Framework), then the immediately following Apache foundation license
  applies, if it gets distributed via the "http://www.ooRexx.org" projtect , then the
  CPL 1.0 license which follows thereafter.


------------------------ Apache Version 2.0 license -------------------------
   Copyright 2006 Rony G. Flatscher

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
-----------------------------------------------------------------------------



------------------------ Common Public License 1.0 --------------------------
   Copyright 2005 Rony G. Flatschre

   This program and the accompanying materials are made available under the
   terms of the Common Public License v1.0 which accompanies this distribution.

   It may also be viewed at: http://www.opensource.org/licenses/cpl1.0.php
-----------------------------------------------------------------------------

*/

      -- load the BSF4Rexx functions and start a JVM, if necessary
   if rxFuncQuery("BSF") = 1 then   -- BSF() support not registered yet ?
   do
      call registerBSFFunctions  -- register  BSF-functions, load JVM
   end


   /*
      pre-registered classes (in the BSF-registry) to make it easier to
      indicate the most important Java class-objects from the ooRexx side;

      just send the Java datatype name (bool, byte, char, int, long,
      float, double, or the Java class names representing the primitive datatypes
      as objects: boolean.class, byte.class, character.class, integer.class,
      long.class, float.class, double.class; in addtion the important Java class
      object names: class.class, object.class, method.class, array.class,
      string.class, system.class) to the directory ".bsf4rexx" and use the
      returned object as argument for the Java side
   */
   .local~bsf4rexx=.directory~new
   .local~b4r=.bsf4rexx             -- create shorter alias for .BSF4Rexx-directory

   arr=.array~of( ,
      "Class.class",      'Class.class'        ,,   -- a Java Object class
      "Object.class",     'Object.class'       ,,   -- a Java Object class
      "Method.class",     'Method.class'       ,,   -- a Java Object class
       ,
      "Array.class",      'Array.class'        ,,   -- a Java Object class
      "String.class",     'String.class'       ,,   -- a Java Object class
      "System.class",     'System.class'       ,,   -- a Java Object class
       ,
      "boolean",          'boolean.class'      ,,   -- primitive
      "boolean.class",    'Boolean.class'      ,,   -- a Java Object class
      "byte",             'byte.class'         ,,   -- primitive
      "byte.class",       'Byte.class'         ,,   -- a Java Object class
      "char",             'char.class'         ,,   -- primitive
      "Character.class" , 'Character.class'    ,,   -- a Java Object class
      "double",           'double.class'       ,,   -- primitive
      "Double.class",     'Double.class'       ,,   -- a Java Object class
      "float",            'float.class'        ,,   -- primitive
      "Float.class",      'Float.class'        ,,   -- a Java Object class
      "int",              'int.class'          ,,   -- primitive
      "Integer.class",    'Integer.class'      ,,   -- a Java Object class
      "long",             'long.class'         ,,   -- primitive
      "Long.class",       'Long.class'         ,,   -- a Java Object class
      "short",            'short.class'        ,,   -- primitive
      "Short.class",      'Short.class'        ,,   -- a Java Object class
      "void",             'void.class'         ,,   -- primitive
      "Void.class",       'Void.class'         )    -- a Java Object class

  do i=1 to arr~items by 2
     .bsf4rexx~setentry(arr[i], .bsf_reference~new(arr[i+1]))
  end

      -- Java classes for "String" and those representing the primitive datatypes
      -- creates ooRexx proxy classes
   arr=.array~of( ,
      'java.lang.String'            ,,   -- a Java Object class
      'java.lang.Boolean'           ,,   -- a Java Object class
      'java.lang.Byte'              ,,   -- a Java Object class
      'java.lang.Character'         ,,   -- a Java Object class
      'java.lang.Double'            ,,   -- a Java Object class
      'java.lang.Float'             ,,   -- a Java Object class
      'java.lang.Integer'           ,,   -- a Java Object class
      'java.lang.Long'              ,,   -- a Java Object class
      'java.lang.Short'             )    -- a Java Object class

  do i=1 to arr~items
--     .bsf~bsf.import(arr[i], arr[i])  -- make these classes accessible as if they were ooRexx classes
     .bsf~bsf.import(arr[i])  -- make these classes accessible as if they were ooRexx classes
  end

.bsf~bsf.import("java.lang.System")

-- box primitive datatypes into their Java classes counterparts
-- BOolean, BYte, C, D, F, I, L, SHort, STring

:: routine box  public
   parse upper arg type +2, val

   select
     when abbrev(type, "BO") then
          do
             if val=.true | val~left(1)="T" then val="true"
                                            else val="false"
             return .java.lang.Boolean~new(val)
          end

     when abbrev(type, "BY") then return .java.lang.Byte~new(val)

     when abbrev(type, "C")  then return .java.lang.Character~new(arg(2)) -- use val verbatimly (retain mixed case)
     when abbrev(type, "D")  then return .java.lang.Double~new(val)

     when abbrev(type, "F")  then return .java.lang.Float~new(val)
     when abbrev(type, "I")  then return .java.lang.Integer~new(val)
     when abbrev(type, "L")  then return .java.lang.Long~new(val)
     when abbrev(type, "SH") then return .java.lang.Short~new(val)
     when abbrev(type, "ST") then return .java.lang.String~new(arg(2))    -- use val verbatimly (retain mixed case)

     otherwise return arg(2)
   end

-- unbox primitive datatype from their Java class counterpart, where they are embedded
-- BOolean, BYte, C, D, F, I, L, SHort, STring
:: routine unbox public
   use arg o

     -- extract class name from proxy's objectName (e.g. "java.lang.String@a3b0cdef")
   parse upper value o~objectname with . "." . "." className "@" .

     -- if not a wrapper class return object unchanged
   if pos(className, "BOOLEAN BYTE CHARACTER DOUBLE FLOAT INTEGER LONG SHORT STRING")=0 then return o

     -- toString() would return instead of "1"/"0" the values "true"/"false"
   if abbrev(className, "BO")  then return o~booleanValue

   return o~toString -- Java string representation suffices for ooRexx



-- register BSF() external functions, assume that Java needs to be loaded
:: routine registerBSFFunctions

   call rxFuncAdd "BsfLoadFuncs", "BSF4Rexx", "BsfLoadFuncs"
   signal on syntax
   call BsfLoadFuncs    -- register all the BSF4Rexx functions ...
syntax:
   -- signal off syntax
   if var("SIGL") then
   do
      say "BSF.CLS: SIGL="sigl "= line # with exception, line: ["sourceline(sigl)"]" -- nur bei exception
      say "         exception:" condition("C")
      say "         traceback:"
      do line over condition("o")~traceback
          say "                  " line
      end
   end

   if rxFuncQuery("BSFLOADJAVA") = 0 then -- load JVM only, if BSFLoadJava() is registered
   do
      -- .error~say( "... loading JVM ...")
      call BsfLoadJava     -- load JVM
   end
   else
   do
      .error~say("... cannot load the JVM, BSFLoadJava() not registered.")
   end

   return

/* -------------------------------------------------------------
   routine which turns an array into an argument string as expected
   by the Java side, starting with the indicated element
   ------------------------------------------------------------- */
:: ROUTINE makeArgString
   use arg args

   tmpString    = ""
   do i=1 to args~items
      if args[i]=.nil then
         tmpString = tmpString ', "' || .bsf~bsf.null_string || '"'
      else
         tmpString = tmpString ', "' || args[i] || '"'
   END
   return tmpString


/* -------------------------------------------------------------
   routine which mimicks C/Java's "x ? a : b;"
   if arg(1) is true, returns second argument, else third
   ------------------------------------------------------------- */
:: ROUTINE choose
   if arg(1)=.true then return arg(2)
                   else return arg(3)


/* -------------------------------------------------------------
   floating class method meant for attaching to imported Java
   classes (for which an Object Rexx proxy class is created)
   to construct an instance (a "bean") with strict arguments (i.e.
   type information are given for each argument; arguments are
   passed to BSF which uses them to create the bean instance
   from the given Java class, ---rgf, 2003-09-05
   ------------------------------------------------------------- */
::METHOD newStrict
  USE ARG javaClass, args

  instance=.bsf_reference~new("uninitialized BSF-object!")

        -- create an instance of javaClass and register it with BSF,
        -- leave beanName empty, this way we get a unique name from Java
/* 2004-02-22
  if args~class = .array then tmpStr=makeArgString(args)
                         else tmpSTr=makeArgString(ARG(1, "Array"))

  code = 'beanName=BSF( "registerBeanStrict", , "' || self~java_path || '"' tmpStr ")"
*/

   if args~class = .array then tmpArr=args            -- array of arguments
                          else tmpArr=arg(1, "Array") -- create an array from arguments

   code = 'beanName=BSF( "registerBeanStrict", , "' || self~bsf.FullJavaClassName || '"'
   do i=1 to tmpArr~items
      a.i=choose(tmpArr[i]=.nil, .bsf~bsf.null_string, tmpArr[i])
      code=code ", a."i
   end
   code=code ")"

  -- say "newStrict, code: ["code"]..."


  signal on any         -- trap any exceptions
  interpret code        /* execute the code, calling bsf() with 'registerBean'  */

        -- assign beanName to object, use the Java supplied name
  instance~ObjectName=beanName

  return instance

any: raise propagate  -- raise the exception in caller/invoker on the Rexx side



/* -------------------------------------------------------------
   routine which checks whether data returned from the Java side
   represents a Java object; if so, an Object Rexx proxy is created
   and returned, else the received string value
   ------------------------------------------------------------- */
-- :: ROUTINE bsf.checkResult public
:: ROUTINE bsf.wrap public
   parse arg data

   if data=.bsf~bsf.null_string then return .nil

        /* if an object was returned, it has been registered on the Java side;
           therefore reuse, if possible or, create an Object Rexx proxy of it;
           first check whether the name was constructed on the Java side        */
  if pos("@", data)>0 then      -- is this a beanName, created by Java?
  do
     parse var data before '@' after
     if before <> "" then
     do
        if datatype(after, "X") then    -- o.k. second part a hexadecimal value?
        do
           if pos(" ", after)=0 then    -- o.k. no space there?
           do
              -- create an Object Rexx proxy object for the Java object

              dimension = verify(data, "[")-1  -- is it a Java array?
              -- if dimension>0 & dimension<6 then
              if dimension>0 then   -- rgf, 2003-02-09, no upper limit anymore
                 return .bsf_array_reference~new(data, dimension) -- return a BSF_ARRAY proxy

              return .bsf_reference~new(data)   -- return a BSF_REFERENCE prox
           end
        end
     end
  end

  return data           -- assume the string of a primitive type




/* *************************************************************
   Class:       IBM's "Bean Scripting Framework"
   Purpose:     the proxy super-class for the Java classes
   ************************************************************* */
::CLASS BSF PUBLIC

/* -------------------------------------------------------------
   class method which merely initializes the class attribute "bsf.null_string"
   to its default value
   ------------------------------------------------------------- */
::METHOD init           CLASS
  EXPOSE bsf.null_string bsf.FullJavaClassName bsf.JavaClassObjectProxies bsf.JavaClassObjects
  bsf.null_string=".NIL"
  bsf.FullJavaClassName=.nil     -- this is an Object Rexx path, hence no fully
                         -- qualified Java class available

  bsf.JavaClassObjectProxies=.directory~new
  bsf.JavaClassObjects      =.directory~new


/* -------------------------------------------------------------
   class method to allow invoking static methods via ooRexx
   ------------------------------------------------------------- */
::method unknown class
-- say self~string"::unknown" "self~bsf.FullJavaClassName="self~bsf.FullJavaClassName

-- say "BSF::unknown::class: self~defaultname="self~defaultName "self~objectName="self~objectName
-- say "BSF::unknown::class: self~bsf.FullJavaClassName="self~bsf.FullJavaClassName

  signal on syntax name any
  -- if bsf.getJavaClassObject[self~bsf.FullJavaClassName]=.nil then   -- no fully qualified Java class ("path") given ?
  if self~bsf.FullJavaClassName=.nil then   -- no fully qualified Java class ("path") given ?
  do
      -- Object "object" does not understand message "message"
     raise syntax 97.1 array(self~string, arg(1))
  end

     -- forward unknown message to the BSF proxy object representing the Java class object,
     -- this way it becomes possible to access the Java static (class) methods quite easily

-- ooo=.bsf~bsf.getJavaClassObjectProxy(self~bsf.FullJavaClassName)
-- say "---> .BSF::U... ooo=" (ooo) (ooo~objectname)
-- say "---> .BSF::UNKOWN:CLASS: arg(1)="arg(1) "arg(2)="arg(2) "arg(2)~items="arg(2)~items "<--- <--- <---" ooo

--  forward to (.bsf~bsf.getJavaClassObjectProxy(self~bsf.FullJavaClassName))
   -- rgf, 2005-06-11: use the Java class proxy object to dispatch the message
  forward to (.bsf~bsf.getJavaClassObjectProxy(self~bsf.FullJavaClassName)) MESSAGE (arg(1)) ARGUMENTS (arg(2))

  any: raise propagate              -- raise exception in caller

::method bsf.jco attribute class private -- contains the BSF proxy to the Java Class Object (JCO) in
                                         -- the BSF registry, such that it does not get garbage collected!


/* Utility method to allow storing the jco-object in an attribute as well as the full Java class name
 * this class represents.
*/
::method bsf.setBSF4RexxInfos class
  expose bsf.jco bsf.FullJavaClassName
  if arg(1)<>self | arg()<>3 then return .false

  bsf.jco=arg(2)
  bsf.FullJavaClassName=arg(3)

  return .true

/* -------------------------------------------------------------
   class method allows for creating an ooRexx proxy class for a Java class
   which is made available to this process by placing it (optionally) into the
   .local environment, unless the second argument is explicitly .nil
   ------------------------------------------------------------- */
::METHOD bsf.import class
--  USE ARG classname, bsf.FullJavaClassName
  USE ARG java_path, classname   -- 2005-06-05, changed argument sequence

--  say "BSF.CLS: java_path="pp(java_path) "classname="pp(className)

  bNoClassName=(arg(2, "Omitted") | className="" | .nil=className)

  signal on syntax name any


  o=.bsf~bsf.JavaClassObjects[java_path] -- try to get the ooRexx class object proxy
  -- o=bsf.JavaClassObjectProxies[java_path]  -- do we have the Java class imported already, if so return proxy class object

-- say "... java_path="java_path "className="classname "..." "o="o

  if o<>.nil then return o

      -- let Java create the class object, which we can refer to
  -- jco=.bsf4rexx~class.class~forName(java_path)  -- get Java class object (can be a BSF_Reference !)
  -- jco=bsf("loadClass", java_path)  -- get Java class object (can be a BSF_Reference !)
  jco=bsf.loadClass(java_path)  -- get Java class object (can be a BSF_Reference !)

-- say "BSF.CLS: jco="pp(jco) "jco~objectName="pp(jco~objectName)


  .bsf~bsf.JavaClassObjectProxies[java_path]=jco     -- save jco in directory



-- say "/// java_path=["java_path"]" "self="self
-- say "/// java_path=["java_path"], classname=["classname"] bNoClassName=["||bNoClassName"], jco=["jco"]..."

      -- now create proxy class object
  -- rgf, 2005-08-28 if bNoClassName then
  -- rgf, 2005-08-28: always use the fully qualified java class name, can be retrieved
  --                  via PARSE VAR new_class "The" java_class_name "class"
     new_class = .BSF~subclass(java_path) -- create a subclass of .BSFClass (a proxy ooRexx class)
     new_class~bsf.setBSF4RexxInfos(new_class, jco. java_path)
     -- new_class~bsf.jco=jco   -- save BSF proxy to avoid garbage collecting the Java Class Object proxy!
     -- new_class~bsf.FullJavaClassName=java_path -- rgf, 2005-08-28, save fully qualified Java path name
     new_class~objectName=jco~objectname      -- now assign beanName
  -- rgf, 2005-08-28else
  -- rgf, 2005-08-28new_class = .BSF~subclass(classname) -- create a subclass of .BSFClass (a proxy ooRexx class)

-- say "... new_class~defaultName="new_class~defaultName "new_class~objectName="new_class~objectName

  .bsf~bsf.JavaClassObjects[java_path]=new_class    -- save the ooRexx Java class object proxy

      -- attach a "newStrict" constructor method to the Object Rexx class object
  new_class~attachNewStrict( java_path )

      -- create code for the INIT-method of the new class
  method_txt = 'self~init:super("' || java_path || '", ARG(1,"A"))'
  new_class~define('init', method_txt)  -- define the constructor instance method "INIT"

--  if bNoClassName | arg(2)=.nil then say "... about returning:" new_class
  if arg(2)=.nil then return new_class

  -- save the new BSF proxy class in .local to make it available as any other ooRexx class
-- say "---> className="className
  if bNoClassName then className=java_path   --- use the Java class name on the ooRexx side as well
  .local~setentry(classname, new_class) -- put new ooRexx proxy class into the local environment

-- say ">>> java_path="java_path "className="className "about returning:" new_class
  return new_class

  any: raise propagate              -- raise exception in caller


/* -------------------------------------------------------------
   allows to retrieve the fully qualified Java class name, if a
   Java class got imported (encapsulated in an Object Rexx class
   proxy); needs to be a method in order to be able to use Object's
   SETMETHOD
   ------------------------------------------------------------- */
::METHOD attachNewStrict class
  use arg java_path
  if self~superclasses[1]<>.bsf then return   -- not a direct subclass of "BSF",
                                              -- hence not from "IMPORT"
  self~setmethod("newStrict", .methods["NEWSTRICT"])
  if self~bsf.FullJavaClassName=.nil then    -- rgf, 2005-08-28
     self~bsf.FullJavaClassName=java_path    -- save fully qualified Java class name


/* -------------------------------------------------------------
   allows to retrieve the fully qualified Java class name, if a
   Java class got imported (encapsulated in an Object Rexx class
   proxy); not directly accessible from outside the class
   ------------------------------------------------------------- */
::method bsf.FullJavaClassName class  -- can be read from any object
  expose bsf.FullJavaClassName
  return bsf.FullJavaClassName

::method  "bsf.FullJavaClassName=" class private  -- can only be set via "self"
  expose  bsf.FullJavaClassName
  use arg bsf.FullJavaClassName


/* -------------------------------------------------------------
   class method which returns the BSF proxy object wrapping
   a Java class object
   ------------------------------------------------------------- */
::method bsf.getJavaClassObjectProxy class
  use arg javaClassName

  o=self~bsf.JavaClassObjectProxies[javaClassName]   -- try to get proxy for Java class object
  if o=.nil then           -- no proxy for Java class object yet, create one
  do
     .bsf~bsf.import(javaClassName) -- import the Java class
     o=self~bsf.JavaClassObjectProxies[javaClassName]   -- try to get proxy for Java class object
  end
  return o        -- return the BSF Java class proxy



/* -------------------------------------------------------------
   directory of BSF proxy objects referring to the Java class object
   ------------------------------------------------------------- */
::method bsf.JavaClassObjectProxies  ATTRIBUTE CLASS

/* -------------------------------------------------------------
   directory of ooRexx BSF proxy objects classes having new() and
   newStrict() defined on them
   ------------------------------------------------------------- */
::method bsf.JavaClassObjects        ATTRIBUTE CLASS

/* -------------------------------------------------------------
   class attribute method stores and allows access to the string
   which is used to indicate "null" for the Java side; upon return
   of values from Java, a Java "null" will be represented via
   Object Rexx' .nil object
   ------------------------------------------------------------- */
::METHOD bsf.null_string    ATTRIBUTE CLASS



/* -------------------------------------------------------------
   method to construct an instance (a "bean"); arguments are
   passed to BSF which uses them to create the bean instance
   from the given Java class
   ------------------------------------------------------------- */
::METHOD init
  USE ARG javaClass, args

        -- create an instance of javaClass and register it with BSF,
        -- leave beanName empty, this way we get a unique name from Java
--  code = 'beanName=BSF( "registerBean", , "' || javaClass || '"' makeArgString(ARG(2, "Array")) ")"

  /* 2004-02-22
  if args~class = .array then tmpStr=makeArgString(args)
                         else tmpSTr=makeArgString(ARG(2, "Array"))

--  code = 'beanName=BSF( "registerBean", , "' || javaClass || '"' makeArgString(args) ")"
  code = 'beanName=BSF( "registerBean", , "' || javaClass || '"' tmpStr ")"
  */


   if args~class = .array then tmpArr=args            -- array of arguments
                          else tmpArr=arg(2, "Array") -- create an array from arguments

   code = 'beanName=BSF( "registerBean", , "' || javaClass || '"'
   do i=1 to tmpArr~items
      a.i=choose(tmpArr[i]=.nil, .bsf~bsf.null_string, tmpArr[i])
      code=code ", a."i
   end
   code=code ")"

  -- say "init, code: ["code"]..."

  signal on any         -- trap any exceptions
  interpret code        /* execute the code, calling bsf() with 'registerBean'  */

        -- assign beanName to object, use the Java supplied name
  self~ObjectName=beanName

-- say "---> BSF.init  : bean=["||beanName||"]"  -- 2003-01-05, ---rgf
  return self

any: raise propagate  -- raise the exception in caller/invoker on the Rexx side

/* -------------------------------------------------------------
   destructor method (called by Object Rexx when object gets
   destroyed) to unregister the bean from the BSF registry; this
   then allows the JVM to garbage collect that Java object as well
   ------------------------------------------------------------- */
::METHOD uninit
-- say "---> BSF.uninit: bean=["||self~objectname||"]" -- 2003-01-05, ---rgf

  return BSF("unregisterBean", self~objectname)


/* -------------------------------------------------------------
   unknown method (gets called by Object Rexx whenever an unknown
   message was sent to an instance of this class); forwards the
   message to BSF including all of the received arguments
   ------------------------------------------------------------- */
::METHOD unknown
  USE ARG name, args

  signal on syntax name any         -- trap any exceptions

--  say " unknown(): name=["name"], arg()="arg() "arg(1)="arg(1) "arg(2)="arg(2)
-- say "            self~string=["self~string"], self~objectname="self~objectname bsf("invoke", self~string, "toString")

-- say "arg(2)="arg(2)~string
if args~class=.array then
do
--    say "            args~items="args~items
--   if args~items>0 then say "            args[1]="args[1]

--    say ' right(name,1)="=" ' (right(name,1)="=")
     if (right(name,1)="=") then -- field assignment?
     do
--    say "BINGO!"
        items=args~items
        parse var name tmpName "=" . -- extract message name without '='
--    say "... tmpName=["tmpName"]"
        if items=1 then             -- arg(1)=value to assign to field
           return self~bsf.setFieldValue(tmpName, args[1])
        else if items=2 then        -- assume arg(1)=typeindicator, arg(2)=value
           return self~bsf.setFieldValueStrict(tmpName, args[1], args[2])
     end
end



-- say "BSF.CLS::unknown, name:" name "for" "'"self~objectname"'" "class:" self~class

/* 2004-02-22: can be longer than 250 chars which causes Object Rexx to bomb!
   code = 'call BSF "invoke", "' || self~objectname || '", "' || name || '"' makeArgString(args)
*/

   code = 'call BSF "invoke", "' || self~objectname || '", "' || name || '"'
   do i=1 to args~items
      a.i=choose(args[i]=.nil, .bsf~bsf.null_string, args[i])

      code=code ", a."i
   end

-- say "unknown, code: ["code"]... a.1="a.1

  signal on any         -- trap any exceptions
  interpret code        -- execute this dynamically created Rexx string
  if var("RESULT") then return bsf.wrap(result)   -- check whether primitive type or Java object
                   else return .nil                      -- nothing came back, i.e. Java returned a true "null"

any:
-- say "---> ---> ---> Ooops! BSF.CLS, SIGL="SIGL "condition()="condition()
raise propagate  -- raise the exception in caller/invoker on the Rexx side




/* -------------------------------------------------------------
   the following methods implement the individual BSF-calls as
   methods, allowing for invoking them in the OO style with the
   help of messages;

   "registerBean" is not made available explicitly, it is
   implicitly part of the creation of a new proxy object (see
   "INIT" method above)

   "unregisterBean" is not made available explicitly, it is
   implicitly part of the destruction of objects (see the
   "UNINIT" methods above)

   "getStaticValue" is not made available to instances as they
   are able to get (and even set) public (static) fields via
   "getFieldValue" resp. "setFieldValue"
   ------------------------------------------------------------- */
--   instance methods

::METHOD bsf.class      -- method to return the BSF Java class object proxy of this Java object proxy
   PARSE VALUE self~objectname WITH java_path "@" .
   return .bsf~bsf.getJavaClassObjectProxy(java_path)


:: METHOD bsf.addEventListener  /* add event listener to the Java object*/
   use arg eventSetName, filter, text
   return BSF( "addEventListener", self~objectname, eventSetName, filter, text )

:: METHOD bsf.exit              /* exit Rexx                    */
   FORWARD MESSAGE ("BSF.EXIT") TO (self~class)

:: METHOD bsf.invoke
   parse arg methodName
   -- create UNKNOWN format (first argument is unknown method name)
   return self~unknown(methodName, arg(2, 'A'))


::METHOD bsf.invokeStrict
  USE ARG name          -- name of method to be invoked
  args=arg(2, 'A')

  -- 2004-02-22
  -- code = 'call BSF "invokeStrict", "' || self~objectname || '", "' || name || '"' makeArgString(args)

   code = 'call BSF "invokeStrict", "' || self~objectname || '", "' || name || '"'
   do i=1 to args~items
      a.i=choose(args[i]=.nil, .bsf~bsf.null_string, args[i])
      code=code ", a."i
   end

  -- say "invokeStrict, code=["code"]..."

  signal on any         -- trap any exceptions
  interpret code        -- execute this dynamically created Rexx string
  if var("RESULT") then return bsf.wrap(result)   -- check whether primitive type or Java object
                   else return .nil                      -- nothing came back, i.e. Java returned a true "null"
any: raise propagate  -- raise the exception in caller/invoker on the Rexx side


:: METHOD bsf.getFieldValue
   parse arg fieldName
   return bsf.wrap( bsf("getFieldValue", self~objectname, fieldName) )

:: METHOD bsf.getFieldValueStrict -- case-sensitive fieldName !
   parse arg fieldName
   return bsf.wrap( bsf("getFieldValueStrict", self~objectname, fieldName) )


:: METHOD bsf.setFieldValue
   parse arg fieldName
   /* 2004-02-22
   code="res=bsf('setFieldValue', '" || self~objectname || "', '" || fieldName || ,
                 "'" makeArgString(arg(2, 'A')) ")"
   */

   code="res=bsf('setFieldValue', '" || self~objectname || "', '" || fieldName || "'" ,
                 ", arg(2) )"
  --  say "bsf.setFieldValue, code=["code"]..."

   signal on any         -- trap any exceptions
   interpret code
   return res

any: raise propagate  -- raise the exception in caller/invoker on the Rexx side

:: METHOD bsf.setFieldValueStrict
   parse arg fieldName
   /* 2004-02-22
   code="res=bsf('setFieldValueStrict', '" || self~objectname || "', '" || fieldName || ,
                 "'" makeArgString(arg(2, 'A')) ")"
   */

   code="res=bsf('setFieldValueStrict', '" || self~objectname || "', '" || fieldName || "'" ,
                 ", arg(2)" choose(arg()>2, ", arg(3)", "") ")"

   -- say "bsf.setFieldValueStrict, code=["code"]..."

   signal on any         -- trap any exceptions
   interpret code
   return res

any: raise propagate  -- raise the exception in caller/invoker on the Rexx side


:: METHOD bsf.getPropertyValue
   parse arg propertyName, index
   if arg(2, "Omitted") then index = "" -- no index given, set to empty string
   return bsf.wrap( bsf("getPropertyValue", self~objectname, propertyName, index))


:: METHOD bsf.setPropertyValue
   parse arg propertyName, index
   if arg(2, "Omitted") then index = "" -- no index given, set to empty string
   /* 2004-02-22
   code="res=bsf('setPropertyValue', '" || self~objectname || "', '" || propertyName || "', '" ||,
                 index || "'" makeArgString(arg(3, 'A')) ")"
   */

   code="res=bsf('setPropertyValue', '" || self~objectname || "', '" || propertyName || "', '" ||,
                 index || "'"
   tmpArr=arg(3, 'Array')
   do i=1 to tmpArr~items
      a.i=choose(tmpArr[i]=.nil, .bsf~bsf.null_string, tmpArr[i])
      code=code ", a."i
   end
   code=code ")"

   signal on any         -- trap any exceptions
   interpret code
   return res

any: raise propagate  -- raise the exception in caller/invoker on the Rexx side


:: METHOD bsf.setPropertyValueStrict
   parse arg propertyName, index
   if arg(2, "Omitted") then index = "" -- no index given, set to empty string
   /* 2004-02-22
   code="res=bsf('setPropertyValueStrict', '" || self~objectname || "', '" || propertyName || "', '" ||,
                 index || "'" makeArgString(arg(3, 'A')) ")"
   */

   code="res=bsf('setPropertyValueStrict', '" || self~objectname || "', '" || propertyName || "', '" ||,
                 index || "'"
   tmpArr=arg(3, 'Array')
   do i=1 to tmpArr~items
      a.i=choose(tmpArr[i]=.nil, .bsf~bsf.null_string, tmpArr[i])
      code=code ", a."i
   end
   code=code ")"

   signal on any         -- trap any exceptions
   interpret code
   return res

any: raise propagate  -- raise the exception in caller/invoker on the Rexx side



--   class  methods

/* -------------------------------------------------------------
   class method shuts down the Java Virtual Machine (JVM), which started
   this instance of Rexx, this causes Rexx to be shut down as well; the
   Java side waits "waitTime" msecs before terminating the JVM
   ------------------------------------------------------------- */
::METHOD bsf.exit class
  use arg exitValue, waitTime

  if \datatype(exitValue, "Whole Number") then return BSF("exit")
  if \datatype(waitTime,  "W"           ) then return BSF("exit", exitValue)
  return bsf.wrap(BSF("exit", exitValue, waitTime))

/* -------------------------------------------------------------
   class method to have Java sleep for us "sleepTime" seconds; resolution
   to a msec is calculated, although the Java side may deviate up to 15/100 secs
   (we are not talking realtime here ;)
   ------------------------------------------------------------- */
::METHOD bsf.sleep class
  use arg sleepTime
  return bsf.wrap(BSF("sleep", sleepTime))

:: METHOD     bsf.loadClass                 class
   parse arg JavaClassName
   return bsf.wrap( bsf("loadClass", JavaClassName) )

:: METHOD     bsf.lookupBean                class
   parse arg beanName
   return bsf.wrap( bsf("lookupBean", beanName) )

:: METHOD     bsf.pollEventText             class
   parse arg timeOut
   if \datatype(timeOut, "Whole") then return bsf.wrap( bsf("pollEventText"))
   return bsf.wrap( bsf("pollEventText", timeout))    -- with timeout

:: METHOD     bsf.getStaticValue            class
   parse arg className, fieldName
   return bsf.wrap( bsf("getStaticValue", className, fieldName))

:: METHOD     bsf.getStaticValueStrict      class
   parse arg className, fieldName
   return bsf.wrap( bsf("getStaticValueStrict", className, fieldName))

:: METHOD     bsf.postEventText              class
   parse arg eventText
   return bsf("postEventText", eventText)

:: METHOD     bsf.wrapArray                  class
   parse arg beanName
   return bsf.wrap( bsf("wrapArray", beanName) )

:: METHOD     bsf.createArray                class
/*
   tmp="return bsf.wrap(bsf('createArray'"
   do i=1 to arg()
      tmp=tmp ", """ || arg(i) || """"
   end
   tmp=tmp"))"

   signal on any         -- trap any exceptions
   interpret tmp     -- now carry out the Rexx statement
   return

any: raise propagate  -- raise the exception in caller/invoker on the Rexx side

*/
/*
   signal on syntax name any

   co=.bsf4rexx~entry(arg(1)~string)   -- try pre-registered class objects first
   if .nil=co then   -- nope, something else: must be a Java class object prox
   do
      -- get the BSF registered Java class object
      co=.bsf~bsf.getJavaClassObjectProxy(arg(1)~bsf.FullJavaClassName)
   end
*/

   signal on syntax name oops
   co=arg(1)      -- default to first argument
-- say "---> ---> BSF.CLS: co="co
   if co~hasmethod("BSF.FULLJAVACLASSNAME") then
   do
      -- co=.bsf~bsf.getJavaClassObjectProxy(arg(1)~bsf.FullJavaClassName)
      co=.bsf~bsf.getJavaClassObjectProxy(co~bsf.FullJavaClassName)
   end

-- say "---> ---> BSF.CLS: co="co "co~class="co~class
-- say "                   co~hasmethod(...)="co~hasmethod("BSF.FULLJAVACLASSNAME")
-- say  "                  bsf~bsf.getJavaClassObjectProxy(co~bsf.FullJavaClassName)=".bsf~bsf.getJavaClassObjectProxy(co~bsf.FullJavaClassName)

   signal on syntax name any
   tmp="return bsf.wrap(bsf('createArray' , '"co"'"

   do i=2 to arg()
      tmp=tmp ", """ || arg(i) || """"
   end
   tmp=tmp"))"

   signal on any     -- trap any exceptions
   interpret tmp     -- now carry out the Rexx statement
   return

any: raise propagate -- raise the exception in caller/invoker on the Rexx side

   -- message "Bsf.FullJavaClassName" not known
oops: signal continue




:: METHOD     bsf.wrapEnumeration            class
   parse arg beanName
   return bsf.wrap( bsf("wrapEnumeration", beanName) )

   -- set the string which from now on represents the Java "null" on both sides,
   -- the Rexx and the Java side
:: METHOD     bsf.setRexxNullString         class
   parse arg newValue
   .bsf~bsf.null_string = newValue                  -- set class attribute to the new value
   return bsf("setRexxNullString", newValue)    -- set the Java side to the new value



/* *************************************************************
   Class:       BSF_REFERENCE
   Purpose:     the proxy class for objects returned by the Java side;
                objects of this class, if destroyed, cause via its superclass
                to remove one from the appropriate instance counter of the
                BSF registry
   Restriction: Should only be used by classes/routines in this file

   ************************************************************* */
::CLASS BSF_REFERENCE subclass BSF

/* -------------------------------------------------------------
   method to construct an instance (a "bean"); as the object was returned
   from the Java side, just remember its name, but do not call BSF's init
   again! This proxy object merely allows for sending messages through it
   to the Java object as well. If it goes out of scope, it will be garbage
   collected by Object Rexx which in turn deregisters the object from the
   Java side, decrementing the reference counter there
   ------------------------------------------------------------- */
::METHOD init
   use arg beanName
   self~ObjectName=beanName



/* *************************************************************
   Class:       BSF_REFERENCE
   Purpose:     allows to wrap up Java objects as Object Rexx objects;
                if instances are destroyed, then no derigistration from
                the BSF registry takes place
   restriction: no check is done, whether the given beanName is really
                in the BSF-registry
   ************************************************************* */
::CLASS BSF_PROXY subclass BSF PUBLIC

/* -------------------------------------------------------------
   method to construct an instance (a "bean"); as the object was returned
   from the Java side, just remember its name, but do not call BSF's init
   again! This proxy object merely allows for sending messages through it
   to the Java object as well. If it goes out of scope, it will be garbage
   collected by Object Rexx which in turn unregisters the object from the
   Java side, decrementing the reference counter there
   ------------------------------------------------------------- */
::METHOD init
   use arg beanName
   self~ObjectName=beanName

-- say "---> BSF_PROXY.init  : bean=["||beanName||"]" -- 2003-01-29, ---rgf

   -- make sure that BSF's UNINIT method does not run (it'll deregister
   -- the bean from the BSF registry
::METHOD uninit
  NOP




/* *************************************************************
   Class:       BSF_ARRAY
   Purpose:     the proxy class for array objects returned by the Java side
   ************************************************************* */
::CLASS BSF_ARRAY_REFERENCE     subclass BSF

/* -------------------------------------------------------------
   method to construct an instance (a "bean"); as the object was returned
   from the Java side, just remember its name, but do not call BSF's init
   again! This proxy object merely allows for sending messages through it
   to the Java object as well. If it goes out of scope, it will be garbage
   collected by Object Rexx which in turn unregisters the object from the
   Java side, decrementing the reference counter there
   ------------------------------------------------------------- */
::METHOD init
   expose items dimension indexArray   -- set access to object variable
   use arg beanName, dimension  -- assign "dimension" directly to exposed object variable

   self~ObjectName=beanName
   items=-1                     -- set and indicate, that not calculated as of yet
      -- "indexArray" allows for indexing at's and put's with a Java int array
      -- use exact signature
   indexArray=bsf("invokeStrict", .bsf4rexx~array.class, "newInstance", "o", .bsf4rexx~int, "int", dimension)
   indexArray=.bsf_array_proxy~new(indexArray, 1, dimension)


::METHOD dimension
  expose dimension dimensions

  if arg(1, "Omitted") then return dimension    -- return number of dimensions

  if VAR( dimensions ) = .false then self~getData  -- no data available, get it
  return dimensions[arg(1)]                     -- return entries in this dimension



      -- get the needed information from ArrayWrapper, make it available with Object Rexx attributes
::METHOD getData private
  expose dimensions items

      -- get the array wrapper of this array
  aw=bsf('wrapArray', self~objectname)
  items=bsf('getFieldValue', aw, 'items')       -- get calculated 'items' value
  dimArr=bsf('getFieldValue', aw, 'sizeOfDimension') -- get the Java array containing the dimensions

  dimensions=.array~new                         -- create an Object Rexx array
  do i=1 to bsf('arrayLength', dimArr)          -- get length of 'dimensions' array
     dimensions[i]=bsf('arrayAt', dimArr, i-1)  -- use Java indexing (starting with '0')
  end


  -- Note: This array-proxy is "1"-based (not 0-based as Java), because Object Rexx arrays
  -- are "1", based, therefore subscript adjustments are necessary, before calling the Java side
  -- The indices can be given individually in comma-separated form or in form of a Rexx array
::METHOD at
   expose indexArray
   use arg arg1

      -- adjust Object Rexx 1-based indexing to Java 0-based indexing
   if arg1~class=.array then
   do i=1 to arg1~items
      -- call bsf "arrayPutStrict", indexArray, "int", arg1[i]-1, i-1
      call bsf "arrayPut", indexArray, arg1[i]-1, i-1
   end
   else -- individual enter the indices into the index-array
   do i=1 to arg()
      -- call bsf "arrayPutStrict", indexArray, "int", arg(i)-1, i-1
      call bsf "arrayPut", indexArray, arg(i)-1, i-1
   end
   return bsf.wrap( bsf('arrayAt', self~objectName, indexArray))


  -- allow for referring to array elements as if it was an Object Rexx array
::METHOD "[]"
  FORWARD MESSAGE ("AT")        -- synonym for "AT", use its implementation



  -- Note: This array-proxy is "1"-based (not 0-based as Java), because Object Rexx arrays
  -- are "1", based, therefore subscript adjustments are necessary, before calling the Java side
  -- The indices can be given individually in comma-separated form or in form of a Rexx array
::METHOD put
   expose indexArray
   use arg value, tmpArr

   if value=.nil then value=.bsf~bsf.null_string -- indicate to the Java side a "null"

   if tmpArr~class<>.array then  -- if second argument is not an array, create one
      tmpArr=arg(2, "Array")     -- get rest of arguments as an array

   do i=1 to tmpArr~items        -- adjust indices to Java-style (0-based indices)
      -- call bsf "arrayPutStrict", indexArray, "int", tmpArr[i]-1, i-1
      call bsf "arrayPut", indexArray, tmpArr[i]-1, i-1
   end

   call bsf 'arrayPut', self~objectName, value, indexArray


  -- Note: This array-proxy is "1"-based (not 0-based as Java), because Object Rexx arrays
  -- are "1", based, therefore subscript adjustments are necessary, before calling the Java side
  -- The indices can be given individually in comma-separated form or in form of a Rexx array
::METHOD putStrict
   expose indexArray
   use arg type, value, tmpArr

   if value=.nil then value=.bsf~bsf.null_string -- indicate to the Java side a "null"

   if tmpArr~class<>.array then  -- if second argument is not an array, create one
      tmpArr=arg(2, "Array")     -- get rest of arguments as an array

   do i=1 to tmpArr~items        -- adjust indices to Java-style (0-based indices)
      -- call bsf "arrayPutStrict", indexArray, "int", tmpArr[i]-1, i-1
      call bsf "arrayPut", indexArray, tmpArr[i]-1, i-1
   end

   call bsf 'arrayPutStrict', self~objectName, type, value, indexArray



  -- allow for referring to array elements as if it was an Object Rexx array;
  -- using this syntax variant has the following implication: the valueType and the
  -- value itself must be given as *one* string value; the first word is the valueType
  -- the rest the value!    e.g.: xyz[1, 2, 3] = "String Hi, there!"
::METHOD "[]="
  FORWARD MESSAGE ("PUT") -- let the PUT-method do the work, 2003-05-10

  /* 2003-05-10, ---rgf
  -- synonym for "PUT", use its implementation
  expose dimension
  parse arg type value, .  -- for this variant extract the type from the rest of the value

  tmpArr=arg(2, "Array")   -- get the rest of the arguments in form of an array
  tmpArrItems=tmpArr~items
  newArr=.array~new
  newArr[1]=type           -- save type information
  newArr[2]=value          -- save value

  do i=3 to 9999           -- assign the index-arguments
     j=i-2
     if j>tmpArrItems then leave
     newArr[i]=tmpArr[j]
  end
  FORWARD MESSAGE ("PUT") ARGUMENTS (newArr) -- let the PUT-method do the work
  */


-- calculates (once) and returns number of individual elements in array
::METHOD items
  expose items  -- set access to object variable
  if items=-1 then self~getData     -- not yet set, get the data from the ArrayWrapper
  return items



-- this method creates a supplier method
::METHOD  SUPPLIER

  aw=bsf('wrapArray', self~objectname)
   -- wrap up the Supplier object, such that it will be kept in the BSF registry as long
   -- as the Object Rexx BSF proxy exists
  return bsf.wrap( bsf('invoke', aw, 'supplier', .true) )



-- this method is needed by "DO ... OVER"
::METHOD MAKEARRAY

  aw=bsf('wrapArray', self~objectname)    -- wrap up the array
  arr=bsf('invoke', aw, 'makearray', .false, .false) -- get
  arr=bsf('arrayAt', arr, 0)              -- get the appropriate Java array

  rexxArr=.array~new
  do i=1 to bsf('arrayLength', arr)          -- get length of 'dimensions' array
     -- rexxArr[i]=bsf('arrayAt', arr, i-1)  -- use Java indexing (starting with '0')
     rexxArr[i]=bsf.wrap(bsf('arrayAt', arr, i-1))  -- use Java indexing (starting with '0')
  end
  return rexxArr



   -- wraps the given Java object and allows for using the array methods on it;
   -- if the reference is not used anymore, it will *not* cause the Java object to be de-registered
   -- used in the initialisation of "BSF_ARRAY" to get primitive int array (for the dimensions) without initialising them as standalone BSF-arrays
   --
::CLASS BSF_ARRAY_PROXY SUBCLASS BSF_ARRAY_REFERENCE
::METHOD init
  use arg beanName, dimension, items         -- name of Java object as registered in the BSF registry

  self~objectName=beanName
  self~dimension =dimension
  self~items     =items

::method dimension attribute
::method items     attribute

::METHOD uninit            -- intercept destructor message, don't remove BSF registry entry

  NOP



::routine bsf.loadClass                      public
   parse arg JavaClassName
   return bsf.wrap( bsf("loadClass", JavaClassName) )



::routine bsf.lookupBean                     public
   parse arg beanName
   return bsf.wrap( bsf("lookupBean", beanName) )

::routine bsf.unregisterBean                 public
  parse arg beanName
  return bsf("unregisterBean", beanName)

::routine bsf.pollEventText                  public
   parse arg timeOut
   if \datatype(timeOut, "Whole") then return bsf.wrap( bsf("pollEventText"))
   return bsf.wrap( bsf("pollEventText", timeout))    -- with timeout

::routine bsf.postEventText                  public
   parse arg eventText
   return bsf("postEventText", eventText)

::routine bsf.getStaticValue                 public
   parse arg className, fieldName
   return bsf.wrap( bsf("getStaticValue", className, fieldName))

::routine bsf.getStaticValueStrict           public
   parse arg className, fieldName
   return bsf.wrap( bsf("getStaticValueStrict", className, fieldName))

::routine bsf.getConstant                    public
   parse arg className, fieldName
   return bsf.wrap( bsf("getStaticValueStrict", className, fieldName))

/*
::routine bsf.createArrayOld                    public
   tmp="return bsf.wrap(bsf('createArray'"
   do i=1 to arg()
      tmp=tmp ", """ || arg(i) || """"
   end
   tmp=tmp"))"

   signal on any     -- trap any exceptions
   interpret tmp     -- now carry out the Rexx statement
   return
any: raise propagate -- raise the exception in caller/invoker on the Rexx side
*/


::routine bsf.createArray                    public


/*
   co=.bsf4rexx~entry(arg(1)~string)   -- try pre-registered class objects first
   if .nil=co then   -- nope, something else: must be a Java class object prox
   do
      -- get the BSF registered Java class object
      co=.bsf~bsf.getJavaClassObjectProxy(arg(1)~bsf.FullJavaClassName)
   end
*/

   signal on syntax name oops
   co=arg(1)      -- default to first argument
   if co~hasmethod("BSF.FULLJAVACLASSNAME") then
   do
      co=.bsf~bsf.getJavaClassObjectProxy(arg(1)~bsf.FullJavaClassName)
   end

   signal on syntax name any
   tmp="return bsf.wrap(bsf('createArray' , '"co"'"

   do i=2 to arg()
      tmp=tmp ", """ || arg(i) || """"
   end
   tmp=tmp"))"

   signal on any     -- trap any exceptions
   interpret tmp     -- now carry out the Rexx statement
   return

any: raise propagate -- raise the exception in caller/invoker on the Rexx side

   -- message "Bsf.FullJavaClassName" not known
oops: signal continue



   /* cheap man's "pretty print": encloses string value in square brackets */
::routine pp   public
  return "[" || arg(1)~string || "]"


   /* VBasic-like iif() function: if arg(1) is .true, then return arg(2), else arg(3)  */
::routine iif  public
  if arg(1)=.true then return arg(2)
  return arg(3)

